<?php

class MGL_InstagramGalleryAdmin {
    
    const STATUS_MESSAGE_UPDATED = 'updated';
    const STATUS_MESSAGE_ERROR = 'error';
    const STATUS_MESSAGE_SUCCESS = 'success';
    
    const INSTAGRAM_SERVER_CONNECTION_TYPE_CURL = 'cUrl';
    const INSTAGRAM_SERVER_CONNECTION_TYPE_FOPEN = 'fopen';
    const INSTAGRAM_SERVER_API_HOST = 'https://api.instagram.com/oauth/access_token';

    public $redirectUri;
    public $instagramAppSettings;
    public $flashMessageBag = array();

    public function __construct(){
        $this->redirectUri = site_url().'/wp-admin/options-general.php?page=mgl-instagram-gallery';

        //Add settings page to Admin menu
        add_action('admin_menu', array( $this, 'addSettingsPageToAdminMenu' ) );
        //Load scripts in MGL Instagram Admin panel
        add_action('admin_enqueue_scripts', array( $this, 'loadSettingsPageCSS' ) );
    }

    public function addSettingsPageToAdminMenu() {  
        add_submenu_page(
            'options-general.php', 
            'Instagram Gallery', 
            'Instagram Gallery', 
            'administrator', 
            'mgl-instagram-gallery', 
            array( $this, 'instagramGallerySettingsPage' ) 
        ); 
    }//addSettingsPageToAdminMenu END

    public function loadSettingsPageCSS() {
        if ( 'settings_page_mgl-instagram-gallery' == get_current_screen()->id) {
            wp_enqueue_style("mgl_instagram_admin", MGL_INSTAGRAM_GALLERY_URL_BASE . "/css/mgl_instagram_admin.css", false, "1.0", "all" );
        }
    }//loadSettingsPageCSS END
    
    public function instagramGallerySettingsPage() {  
        //include('mgl_instagram_admin.php');
        $this->checkServerConnections();
        $this->instagramAppSettings = $this->getInstagramAppSettings();

        if( isset($_POST) ) $this->updateSettings();
        
        $this->instagramAppSettings = $this->getInstagramAppSettings();

        if( isset($_GET['delete_instagram_account']) ) $this->removeInstagramApplication();
        if( isset($_GET['code'])) $this->validateApplicationWithInstagramServer();

        $this->renderSettingsPage();

    }//showInstagramGallerySettingsPage END

    public function updateSettings(){
        $updated = false;

        if( isset( $_POST['mgl_update'] ) ){
            update_option( 'mgl_instagram', $_POST['mgl_instagram'] );
            $updated = true;
        }

        if(isset($_POST['mgl_update_settings'])) {
            $jQueryActive = ( isset($_POST['mgl_instagram_jquery']) ) ? true : false;
            update_option( 'mgl_instagram_jquery', $jQueryActive );
            $updated = true;
        }

        if($updated == true) {
            $messageSettingsSaved = __('Settings saved', MGL_INSTAGRAM_GALLERY_DOMAIN );
            $this->addFlashMessage( $messageSettingsSaved, self::STATUS_MESSAGE_UPDATED );
        }
    }//updateSettings END

    public function addFlashMessage( $message, $messageType ){
        $this->flashMessageBag[] = array( $messageType => $message );
    }//addFlashMessage END

    public function renderFlashMessageBag(){
        foreach( $this->flashMessageBag as $flashMessage ){
            foreach( $flashMessage as $messageType => $message ){
                ?>
                    <div class="<?php echo $messageType; ?>"> 
                        <p><strong><?php echo $message ?></strong></p>
                    </div>
                <?php
            }
        }
        $this->flashMessageBag = array();
    }

    public function removeInstagramApplication(){
        delete_option('mgl_instagram');
        delete_option('mgl_instagram_jquery');
        delete_option('mgl_instagram_userinfo');

        $messageSettingsSaved = __('Application settings removed correctly', MGL_INSTAGRAM_GALLERY_DOMAIN );
        $this->addFlashMessage( $messageSettingsSaved, self::STATUS_MESSAGE_UPDATED );

    }//removeInstagramApplication END

    public function checkServerConnections(){
        $connectionAllowed = false;
        if( $this->isCurlEnabled() ) $connectionAllowed = true;
        if( $this->isFopenEnabled() ) $connectionAllowed = true;

        if( !$connectionAllowed ){
            $message = __('Your server does not allow <strong>cUrl</strong> or <strong>Allow Url Fopen</strong> methods to connect to Instagram Servers. Please enable at less one of them in order to make the plugin work', MGL_INSTAGRAM_GALLERY_DOMAIN );
            $this->addFlashMessage( $messageSettingsSaved, self::STATUS_MESSAGE_ERROR );
        }
    }//checkServerConnections END

    public function isCurlEnabled(){
        if( !function_exists('curl_version') ) return false;
        return true;
    }//isCurlEnabled END

    public function isFopenEnabled(){
        if( !ini_get('allow_url_fopen') ) ini_set('allow_url_fopen', true );
        if( !ini_get('allow_url_fopen') ) { //If it is still false throw a message
            return false;
        }
        return true;
    }//EisFopenEnabled END

    public function getInstagramAppSettings(){
        $defaultInstagramServiceApplicationSettings = array(
            'user'             => '', 
            'client_id'        => '', 
            'client_secret'    => ''
        );
        return get_option('mgl_instagram', $defaultInstagramServiceApplicationSettings);
    }

    public function renderSettingPageTabs( $current = 'instagram-app' ) {
        $tabs = array( 
            'instagram-app' => __('Instagram application',MGL_INSTAGRAM_GALLERY_DOMAIN), 
            'configuration' => __('Configuration',MGL_INSTAGRAM_GALLERY_DOMAIN), 
            'documentation' => __('Documentation',MGL_INSTAGRAM_GALLERY_DOMAIN) 
        );

        echo '<div id="icon-themes" class="icon32"><br></div>';
        echo '<h2 class="nav-tab-wrapper">';
        foreach( $tabs as $tab => $name ){
            $class = ( $tab == $current ) ? ' nav-tab-active' : '';
            echo "<a class='nav-tab$class' href='?page=mgl-instagram-gallery&tab=$tab'>$name</a>";
        }
        echo '</h2>';
    }//renderSettingPageTabs END

    public function renderRegisteredInstagramApplication( $instagramAccount ){
        if($instagramAccount != false && isset($instagramAccount->access_token)) { ?>
        
            <h3><?php _e('Application settings', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></h3>
            <div class="mgl_instagram_account">
                <img src="<?php echo $instagramAccount->user->profile_picture; ?>" alt="<?php echo $instagramAccount->user->username; ?>">
                <?php print_r($instagramAccount->user->id); ?>
                <h4><?php echo $instagramAccount->user->username; ?></h4>
                <p>
                    <?php echo $instagramAccount->user->bio; ?>
                    <a href="<?php echo $instagramAccount->user->website; ?>"><?php echo $instagramAccount->user->website; ?></a>
                </p>
                <p><strong><?php _e('Acces token', MGL_INSTAGRAM_GALLERY_DOMAIN); ?>:</strong> <?php echo $instagramAccount->access_token; ?></p>
                <a href="?page=mgl-instagram-gallery&delete_instagram_account=true"><?php _e('Remove application', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></a>
            </div>
        
        <?php }
    }

    public function renderInstagramAppTab(){
        $instagramAccount = $this->getInstagramAccount();
        ?>
            <div class="wpbody-content">
                <div class="wrap">
                    <h2><?php _e('Instagram application', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></h2>
                    <div class="col_left"> 
                        <p><?php printf(__('You need to register a new Instagram application from %s in order to make this plugin work', MGL_INSTAGRAM_GALLERY_DOMAIN), '<a href="http://instagram.com/developer/clients/register/" target="_blank">'.__('here',MGL_INSTAGRAM_GALLERY_DOMAIN).'</a>'); ?></p> 
                        <p><?php _e('The name and description of the application are not important, just be sure to put the same Website & OAuth redirect_uri written down here', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></p>
                        
                        <form name="mgl_instagram_form" method="post" action="<?php echo site_url().'/wp-admin/options-general.php?page=mgl-instagram-gallery'; ?>"> 
                            <input type="hidden" name="mgl_update" value="Y"> 
                            <table class="form-table">
                                <tr valign="top"> 
                                    <th scope="row"><?php _e('Website', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></th>
                                    <td><?php echo site_url(); ?></td>
                                </tr>
                                <tr valign="top"> 
                                    <th scope="row"><?php _e('OAuth redirect_uri', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></th>
                                    <td><?php echo $this->redirectUri; ?></td>
                                </tr> 
                            </table>
                            <p><?php _e('Fill this two fields with the information of your App', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></p>
                            <table class="form-table">
                                <tr valign="top"> 
                                    <th scope="row"><?php _e('Client ID', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></th>
                                    <td><input class="regular-text" type="text" name="mgl_instagram[client_id]" value="<?php echo $this->instagramAppSettings['client_id']; ?>" /></td>
                                </tr>      
                                <tr valign="top"> 
                                    <th scope="row"><?php _e('Client secret', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></th>
                                    <td><input class="regular-text" type="text" name="mgl_instagram[client_secret]" value="<?php echo $this->instagramAppSettings['client_secret']; ?>" /></td>
                                </tr>    
                            </table>
                            <p><?php _e('Save settings, then you can authorize your app!', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></p>            
                            <p class="submit">
                                <input class="button" type="submit" name="Submit" value="<?php _e('Save settings', MGL_INSTAGRAM_GALLERY_DOMAIN ) ?>" />  
                                <?php if($this->instagramAppSettings['client_id'] != '' && $this->instagramAppSettings['client_secret'] != '') { ?>
                                <a class="button button-primary" href="https://api.instagram.com/oauth/authorize/?client_id=<?php echo $this->instagramAppSettings['client_id']; ?>&redirect_uri=<?php echo $this->redirectUri; ?>&response_type=code"><?php _e('Authorize app', MGL_INSTAGRAM_GALLERY_DOMAIN); ?></a>
                                <?php } ?>
                            </p> 
                        </form> 
                        
                        <?php $this->renderRegisteredInstagramApplication( $instagramAccount ); ?>

                    </div> 
                    <div class="col_right">
                        <a class="mgl_banner" href="http://codecanyon.net/user/MaGeekLab?ref=mageeklab" title="Follow us on CodeCanyon" target="_blank"><img  title="Follow us on CodeCanyon"  alt="Follow us on CodeCanyon" src="<?php echo MGL_INSTAGRAM_GALLERY_URL_BASE.'images/mageeklab_banner_codecanyon.png'; ?>" alt=""></a>
                    </div>
                </div>
            </div>
        <?php
    }//renderInstagramAppTab END

    public function renderConfigurationTab(){
        include('admin-settings-tabs/configuration-tab.php');
    }//renderConfigurationTab END

    public function renderDocumentationTab(){
        include( 'admin-settings-tabs/documentation-tab.php');
    }//renderDocumentationTab END

    public function renderSettingPageCurrentTab( $currentTab ){
        switch ( $currentTab ) {
            case 'configuration':
                $this->renderConfigurationTab();
                break;
            case 'documentation':
                $this->renderDocumentationTab();
                break;
            case 'instagram-app':
            default:
                $this->renderInstagramAppTab();
                break;
        }
    }//renderSettingPageCurrentTab END

    public function renderSettingsPage(){
        $currentTab = ( isset($_GET['tab']) ) ?  $_GET['tab'] : 'instagram-app';
        ?>
            <div class="mgl_wrap">
                <h1><?php _e( 'MaGeek Lab - Instagram Gallery', MGL_INSTAGRAM_GALLERY_DOMAIN ); ?></h1>
                <?php 
                    $this->renderSettingPageTabs( $currentTab ); 
                    $this->renderFlashMessageBag(); 
                    $this->renderSettingPageCurrentTab( $currentTab ); 
                ?>
            </div>
        <?php
    }//renderSettingsPage END


    public function getJsonDataFromInstagramServer( $apiData ){
        $request = new WP_Http;
        $result = $request->request( 
            self::INSTAGRAM_SERVER_API_HOST, 
            array( 
                'method' => 'POST', 
                'body' => $apiData,
                'headers' => array('Accept' => 'application/json' )
            ) 
        );
        if( $result['response']['code'] != 200 ) return false;

        return $result['body'];
    }

    public function validateApplicationWithInstagramServer(){
        $errors = array();
        $code = $_GET['code'];

        $apiData = array(
          'client_id'       => $this->instagramAppSettings['client_id'],
          'client_secret'   => $this->instagramAppSettings['client_secret'],
          'grant_type'      => 'authorization_code',
          'redirect_uri'    => $this->redirectUri,
          'code'            => $code
        );

        $jsonInstagramResponse = $this->getJsonDataFromInstagramServer( $apiData );
        if($jsonInstagramResponse){

            $user = @json_decode($jsonInstagramResponse); 
            if(isset($user->access_token)) {
                $message =  __('<strong>App authorized!</strong> You are ready to go', MGL_INSTAGRAM_GALLERY_DOMAIN);
                $this->addFlashMessage( $message , self::STATUS_MESSAGE_SUCCESS );

                update_option( 'mgl_instagram_userinfo', base64_encode(serialize($user)) );

            } else {
                $message = '<strong>'.__('Error', MGL_INSTAGRAM_GALLERY_DOMAIN).':</strong> '.$user->error_message;
                $this->addFlashMessage( $message , self::STATUS_MESSAGE_ERROR );
            }
        }else{
            $message = __('Error: Imposible to connect with Instagram server', MGL_INSTAGRAM_GALLERY_DOMAIN);
            $this->addFlashMessage( $message , self::STATUS_MESSAGE_ERROR );
        }
    }

    public function getInstagramAccount(){
        $instagramAccount = unserialize( base64_decode( get_option('mgl_instagram_userinfo') ) );
        return $instagramAccount;
    }
}

$mglInstagramGalleryAdmin = new MGL_InstagramGalleryAdmin();
