<?php

class MGL_InstagramGalleryController {

    public function __construct(){

        //Add jQuery to public side of the web
        if(get_option('mgl_instagram_jquery', true) == true) {
            add_action('wp_print_scripts', array( $this, 'enqueueJQueryToPublicPartWeb' ) );
        }

        //Register widgets
        add_action('widgets_init', array( $this, 'requireAndRegisterGalleryWidgets' ) );
        //Add query vars //TODO Cambiar por QUERY args => http://codex.wordpress.org/Function_Reference/add_query_arg
        add_filter( 'query_vars', array( $this, 'mgl_instagram_query_vars' ), 10, 1 );
        //Register session if not
        add_action('init', array( $this, 'register_session' ) );

        //Shortcodes
        add_shortcode('mgl_instagram_user', array( $this, 'galleryShortcodes' ) );
        add_shortcode('mgl_instagram_feed', array( $this, 'galleryShortcodes' ) );
        add_shortcode('mgl_instagram_liked', array( $this, 'galleryShortcodes' ) );
        add_shortcode('mgl_instagram_tag', array( $this, 'galleryShortcodes' ) );
        add_shortcode('mgl_instagram_location', array( $this, 'galleryShortcodes' ) );
        add_shortcode('mgl_instagram_location_search', array( $this , 'renderLocationsID' ) );
    }

    public function requireAndRegisterGalleryWidgets(){

        require_once('widgets/widget-mgl_instagram_user.php');
        require_once('widgets/widget-mgl_instagram_tag.php');
        require_once('widgets/widget-mgl_instagram_feed.php');
        require_once('widgets/widget-mgl_instagram_liked.php');
        require_once('widgets/widget-mgl_instagram_location.php');

        register_widget( 'mgl_Instagram_user' );
        register_widget( 'mgl_Instagram_tag' );
        register_widget( 'mgl_Instagram_feed' );
        register_widget( 'mgl_Instagram_liked' );
        register_widget( 'mgl_Instagram_location' );
    }

    public function enqueueJQueryToPublicPartWeb() {
        if(!is_admin()) {
            wp_enqueue_script("jquery"); 
        }
    }

    public function mgl_instagram_query_vars( $qvars ) {
        array_push( $qvars, 'media' );
        array_push( $qvars, 'title' );
        return $qvars;
    }

    public function register_session(){
        if( !session_id() )
            session_start();
    }

    public function registerStylesAndScripts(){
        // Setup needed libraries
        wp_register_style('mgl_instagram_gallery', plugin_dir_url(__FILE__) . 'css/mgl_instagram_gallery.css');
        wp_register_script('mgl_instagram_gallery_magnific', plugin_dir_url(__FILE__) . 'js/magnific.js');
        wp_register_style('mgl_instagram_gallery_magnific', plugin_dir_url(__FILE__) . 'js/magnific-popup.css');

        wp_register_script('mgl_instagram_gallery_video', plugin_dir_url(__FILE__) . 'js/video.js');
        wp_register_style('mgl_instagram_gallery_video', plugin_dir_url(__FILE__) . 'js/video-js.css');
        wp_register_script('mgl_instagram_gallery_jplayer', plugin_dir_url(__FILE__) . 'js/jquery.jplayer.min.js');
        wp_register_style('mgl_instagram_gallery_jplayer', plugin_dir_url(__FILE__) . 'js/jplayer.blue.monday.css');

        wp_register_script('mgl_instagram_gallery_loader', plugin_dir_url(__FILE__) . 'js/mgl_instagram_loader.js');
        wp_enqueue_script('mgl_instagram_gallery_loader');
    }

    public function galleryShortcodes($atts, $content = null, $tagName) {

        $this->registerStylesAndScripts();
        if( !session_id() ) session_start();

        $atts = $this->sanatizeShortcodeArguments( $atts );

        extract(shortcode_atts(array(
            'number'         => '12',
            'cache'             => 3600,
            'pagination'     => 'true',
            'username'         => '',
            'user_id'         => 0,
            'location_id'    => 0,
            'tag'            => '',
            'cols'             => 4,
            'more_text'     => 'View more',
            'previous_text' => 'Go back',
            'video'         => 'true',
            'cut_text'         => 80,
            'direct_link'    => 'false',
            'disable_js'    => 'false',
            'skin'            => 'default',
            'responsive'    => 'true',
            'debug'            => 'false'
        ), $atts));

        if($video       == 'false') { $video = false; } else { $video = true; }
        if($direct_link == 'false') { $direct_link = false; } else { $direct_link = true; }
        if($disable_js  == 'false') { $disable_js = false; } else { $disable_js = true; }
        if($responsive  == 'false') { $responsive = false; } else { $responsive = true; }
        
        if($debug       == 'true') { $debug = true; } else { $debug = false; }
        if($pagination  == 'false') { $pagination = false; } else { $pagination = true; }

        $args = array(
            'count' => $number,
            'user_id' => $user_id,
            'username' => $username,
            'location_id' => $location_id,
            'tag' => $tag,
            'cache' => $cache,
            'pagination' => $pagination,
            'cols' => $cols,
            'more_text' => $more_text,
            'previous_text' => $previous_text,
            'cut_text' => $cut_text,
            'video' => $video,
            'direct_link' => $direct_link,
            'disable_js' => $disable_js,
            'skin'    => $skin,
            'responsive' => $responsive,
            'debug' => $debug
        );

        $gallery = $this->getGalleryByTagName( $tagName, $args );

        return '<div class="'.$number.' '.$tag.'" >'. $gallery->render() .'</div>';
    }

    public function getGalleryByTagName( $tagName, $args ){
        switch ($tagName) {
            case 'mgl_instagram_user':
                $gallery = new MGL_InstagramGalleryUser( $args );  
                break;
            
            case 'mgl_instagram_feed':
                $gallery = new MGL_InstagramGalleryFeed( $args );
                break;

            case 'mgl_instagram_liked':
                $gallery = new MGL_InstagramGalleryLiked( $args );
                break;

            case 'mgl_instagram_tag':
                $gallery = new MGL_InstagramGalleryTag( $args );
                break;

            case 'mgl_instagram_location':
                $type = 'location';
                $gallery = new MGL_InstagramGalleryLocation( $args );
                break;
            default:
                $gallery = false;
        }

        return $gallery;
    }

    public function sanatizeShortcodeArguments( $args = array() ){
        foreach( $args as $key => $value ){
            if( $value == '' ){
                unset( $args[ $key ] );
            }
        }

        return $args;
    }

    public function renderLocationsID( $atts = array(), $content = null ){
        if( $atts == '' ) $atts = array();
        
        $args = $this->sanatizeShortcodeArguments( $atts );
        $locationSearchObject = new MGL_Instagram_LocationSearch( $args );

    }
}
$mglInstagramGallery = new MGL_InstagramGalleryController();