<?php

abstract class MGL_InstagramGalleryBase {
    //Shortcode attributes
    public  $type,
            $count,
            $userId,
            $username,
            $location_id,
            $tag,
            $cache,
            $pagination,
            $cols,
            $more_text,
            $previous_text,
            $cut_text,
            $video,
            $directLink,
            $disable_js,
            $skin,
            $responsive,
            $debug;

    public $ajaxViewerUrl,
           $instagramAccount,
           $accessToken,
           $galleryId,
           $gallery,
           $nextId,
           $preId;

    abstract protected function getUrlRequestToGetGalleryFromInstagramServer();
    abstract protected function configureNavigation();

    public function __construct( $args ){

        $this->count = $args['count'];
        $this->userId = $args['user_id'];
        $this->username = $args['username'];
        $this->location_id = $args['location_id'];
        $this->tag = $args['tag'];
        $this->cache = $args['cache'];
        $this->pagination = $args['pagination'];
        $this->cols = $args['cols'];
        $this->more_text = $args['more_text'];
        $this->previous_text = $args['previous_text'];
        $this->cut_text = $args['cut_text'];
        $this->video = $args['video'];
        $this->directLink = $args['direct_link'];
        $this->disable_js = $args['disable_js'];
        $this->skin = $args['skin'];
        $this->responsive = $args['responsive'];
        $this->debug = $args['debug'];

        $this->ajaxViewerUrl = plugin_dir_url(__FILE__) . '../single-gallery.php';

        $this->nextId = null;
        $this->prevId = null;

        //CONTROLLER
        $this->galleryId = $this->getGalleryId();

        $this->instagramAccount = $this->getInstagramAccount();
        $this->accessToken = $this->getAccessToken();

        if( $this->accessToken ){
    
            $thisUserId = $this->instagramAccount->user->id;
            if( $this->userId == 0 ){ $this->userId = $thisUserId; }
    
            $this->gallery = $this->getGallery();
        }
    }

    public function getGallery(){
        // Check for pagination in the shortcode
        if( isset($_GET['next']) && isset($_GET['mglGalleryId']) ) {             
            if( $_GET['mglGalleryId'] == $this->galleryId ){
                $this->nextId = $_GET['next'];
            }
        } else { 
            $_SESSION[ $this->galleryId ]['previous_id'] = array();
        }

        $cacheGalleryKey = $this->galleryId . substr( $this->nextId , 0, 10 );

        $cachedGallery = $this->getCachedGallery( $cacheGalleryKey );
        if( $cachedGallery ){ return $cachedGallery; }

        $gallery = $this->getGalleryFromInstagramServer();

        // Save KEY for caching
        if( $gallery->meta->code == '200' && $this->cache > 0 ){
            set_transient( $cacheGalleryKey , base64_encode( serialize( $gallery ) ), $this->cache );
        }

        return $gallery;
    }

    public function getCachedGallery( $galleryKey ){
       $cachedGallery =  get_transient( $galleryKey );
       if ( $cachedGallery === false ) return false;

       return unserialize( base64_decode( $cachedGallery ) );
    }

    public function getGalleryFromInstagramServer(){
        if($this->debug == true) { echo 'New media'; }

        $urlPlain = $this->getUrlRequestToGetGalleryFromInstagramServer();
        $instagramResponse = $this->getInstagramResponse( $urlPlain );

        $gallery = json_decode( $instagramResponse );

        return $gallery;
    }

    public function getInstagramAccount(){
       return unserialize( base64_decode( get_option( 'mgl_instagram_userinfo' ) ) );
    }

    public function getAccessToken(){
        $accessToken = $this->instagramAccount->access_token;

        if( $accessToken == '' ) {
            echo sprintf(__("Error: You don't have an access token, have you configurated correctly the plugin from %s?", 'mgl_instagram_gallery'), '<a href="'.site_url().'/wp-admin/options-general.php?page=mgl-instagram-gallery">'.__('here','mgl_instagram_gallery').'</a>');
            return false;
        }

        return $accessToken;
    }

    public function getGalleryId(){
        return "mgl_ins_" . $this->type . $this->username . $this->userId . $this->tag . $this->location_id . "_n" . $this->count . '_' . $this->cache;
    }

    public function getInstagramResponse( $url ){
        $request = new WP_Http;
        $result = $request->request( 
            $url, 
            array( 
                'method' => 'GET',
                'headers' => array()
            ) 
        );

        if( $result['response']['code'] != 200 ){
            _e( 'Error: Cannot retrive photos from Instagram', 'mgl_instagram_gallery');
            return false;
        }

        return $result['body'];
    }

    public function render(){
        $this->enqueueStylesAndScripts();

        $this->configureNavigation();

        $rand_id = rand(1000, 9999);
        $responsive_class = ( $this->responsive ) ? ' mgl_instagram_gallery_responsive' : '';

        ob_start();

            $loop_index = 0;
            $gallery_data = $this->gallery->data;
            
            echo '<div id="'.$this->galleryId.'_'.$rand_id.'" class="mgl_instagram_gallery cols'.$this->cols.' mgl_instagram_skin_'.$this->skin.$responsive_class.'" mgl-gallery-video="' . (($this->video) ? 'true' : 'false' ). '" mgl-gallery-disablejs="'. ( ($this->disable_js) ? 'true' : 'false' )   .'" mgl-gallery-directlink="'. ( ($this->directLink) ? 'true' : 'false'  ) .'">';
            foreach ($gallery_data as $galleryItem) {             
                if($this->debug == true) { echo '<pre>'; print_r($galleryItem); echo '</pre>'; }

                if($loop_index < $this->count) {
                    $this->renderGalleryItem( $galleryItem );
                    $loop_index++;
                }
            }
            echo '</div>';
            
            // Pagination
            $this->renderPagination();

        $html = ob_get_clean();

        return $html;
    }

    public function renderGalleryItem( $galleryItem ){

        $description = '';
        $descriptionShort = '';
        if(isset($galleryItem->caption->text)) { 
            $description = $galleryItem->caption->text; 
            $descriptionShort = $this->getShortDescription( $description );
        }

        if(isset($galleryItem->likes->count)) { $likes = $galleryItem->likes->count; } else { $likes = 0; }
        if(isset($galleryItem->comments->count)) { $comments = $galleryItem->comments->count; } else { $comments = 0; }
        
        $target = ( $this->directLink ) ? ' target="_blank"' : '';
        $url_big = $this->getGalleryItemUrlBig( $galleryItem, $description );

        include( plugin_dir_path( __FILE__ ) .'/../templates/default/gallery-item.php' );
    }

    public function getGalleryItemUrlBig( $galleryItem ){
        if(isset($galleryItem->caption->text)) { $description = $galleryItem->caption->text; } else { $description = ''; }

        if( !$this->directLink ){
            if( $this->video ){
                if( isset($galleryItem->videos->standard_resolution->url) ){
                    $urlVideo = $galleryItem->videos->standard_resolution->url;
                    return $this->ajaxViewerUrl . '?media=' . base64_encode( $urlVideo ) . '&title=' . base64_encode( $description );
                
                } else {
                    $urlImage = $galleryItem->images->standard_resolution->url;
                    return $this->ajaxViewerUrl . '?media='. base64_encode( $urlImage ) . '&title=' . base64_encode( $description );
                }
            
            } else {
                return $galleryItem->images->standard_resolution->url;
            }
        }

        return $galleryItem->link;
    }

    public function renderPagination(){
        
        if($this->pagination != false && $this->nextId != '') {

            if(isset($_GET['page_id'])) {
                $page = '?page_id='.$_GET['page_id'].'&';
            } else {
                $page = '?';
            }
               
            // Store next_min_id for previous link
            if( isset($this->prevId) && $this->prevId != '' ){
                if( !in_array($this->prevId, $_SESSION[ $this->galleryId ]['previous_id'])) {
                    $_SESSION[ $this->galleryId ]['previous_id'][] = $this->prevId;
                }
            }

            echo '<div class="mgl_instagram_pagination mgl_instagram_skin_'.$this->skin.'">';
            echo '<a class="mgl_instagram_pagination_next" href="'.$page.'mglGalleryId='.$this->galleryId.'&next='.$this->nextId.'">'.$this->more_text.'</a>';
            echo '</div>';
        }
    }

    public function enqueueStylesAndScripts(){

        wp_enqueue_style('mgl_instagram_gallery');
        wp_enqueue_script('mgl_instagram_gallery_magnific');
        wp_enqueue_style('mgl_instagram_gallery_magnific');
        
        if($this->video == true) {
            wp_enqueue_script('mgl_instagram_gallery_video');
            wp_enqueue_style('mgl_instagram_gallery_video');
            wp_enqueue_script('mgl_instagram_gallery_video');
            wp_enqueue_style('mgl_instagram_gallery_video');
        }

        wp_enqueue_script('mgl_instagram_gallery_loader');
    }

    public function getShortDescription( $string ) {
        if($this->cut_text == 0) { return ''; }
        return (strlen($string) > $this->cut_text ) ? substr( $string, 0, $this->cut_text - 3 ).'...' : $string;

    }
}